ProgramSpec

Struct ProgramSpec 

pub struct ProgramSpec {
    pub name: String,
    pub src: String,
    pub device: DeviceSpec,
    pub ast: Arc<UOp>,
    pub global_size: Option<[usize; 3]>,
    pub local_size: Option<[usize; 3]>,
    pub vars: Vec<Variable>,
    pub var_names: Vec<String>,
    pub globals: Vec<usize>,
    pub outs: Vec<usize>,
    pub ins: Vec<usize>,
    pub buf_count: usize,
}
Program specification containing source code and metadata.

This is returned by Renderer::render() and consumed by Compiler::compile(). It bridges the gap between UOp graphs and compiled executables.

Tinygrad Alignment

Buffer metadata (globals, outs, ins) matches Tinygrad’s Program class:

  • globals: Buffer indices from DefineGlobal ops
  • outs: Output buffer indices (written by STORE ops)
  • ins: Input buffer indices (read by LOAD ops)

Fields

name: String

Kernel name (for debugging/profiling)

src: String

Generated source code (LLVM IR, CUDA C, Metal, WGSL, etc.)

device: DeviceSpec

Device specification

ast: Arc<UOp>

Original AST (for cache key construction via hash consing)

global_size: Option<[usize; 3]>

Global work size (for GPU backends, None for CPU)

local_size: Option<[usize; 3]>

Local work size (for GPU backends, None for CPU)

vars: Vec<Variable>

Variable list (for symbolic shapes/strides)

var_names: Vec<String>

Variable names in order for populating vars array at runtime. Includes thread_id at the end if threading is enabled.

globals: Vec<usize>

Global buffer indices (from DefineGlobal argument values). Matches Tinygrad’s globals field.

outs: Vec<usize>

Output buffer indices (written by STORE ops). Matches Tinygrad’s outs field.

ins: Vec<usize>

Input buffer indices (read by LOAD ops, excluding outputs). Matches Tinygrad’s ins field.

buf_count: usize

Number of buffer arguments (for CIF construction at compile time).

Implementations

Source

impl ProgramSpec

pub fn new( name: String, src: String, device: DeviceSpec, ast: Arc<UOp>, ) -> ProgramSpec

Create a new program specification.

pub fn add_var(&mut self, var: Variable)

Add a variable to the program.

pub fn set_work_sizes(&mut self, global: [usize; 3], local: [usize; 3])

Set work sizes for GPU execution.

pub fn set_var_names(&mut self, var_names: Vec<String>)

Set variable names for populating vars array at runtime.

pub fn set_buffer_metadata( &mut self, globals: Vec<usize>, outs: Vec<usize>, ins: Vec<usize>, )

Set buffer metadata (globals, outs, ins).